#include <stdio.h>
#include <string.h>
#include <nds/memory.h>
#include <nds/arm9/console.h>
#include "inout.h"
#include "cardme.h"
#include "menu.h"
#include "smap.h"
#include "Gdefines.h"

void backupEEP(uint8* dest)
{
    clearIt();
    WAIT_CR &= ~0x0880;

    consolePrintSet(0,start_line);
    consolePrintf("Reading EEPROM");
    int type = cardmeGetType();    //-1    0    1    2      3      get type and size of eeprom
    int length = cardmeSize(type); //0, 8192, 512, 8192, 256*1024
    if((type == 1) || (type == 2))
    {
        cardmeReadEeprom(0, dest, length, type); //copy eeprom to sram
        ref_sramID(); //refresh SRAM id string to verify copy OK
        consolePrintSet(0,(start_line+1));
        consolePrintf("Backup Finished");
        return;
    }
    else
    {
        consolePrintSet(0,start_line);
        consolePrintf("Backup Failed!\n"
                      "eeprom size invalid!");
        return;
    }
    return;
}

void restoreEEP(uint8* src)
{
    consolePrintSet(0,start_line);
    WAIT_CR &= ~0x0880;

    consolePrintf("Copying to EEPROM");
    int type = cardmeGetType();    //-1    0    1    2      3      get type and size of eeprom
    int length = cardmeSize(type); //0, 8192, 512, 8192, 256*1024
    if((type == 1) || (type == 2))
    {
        cardmeWriteEeprom(0, src, length, type); //copy FF buffer to EEPROM
        ref_eeprID(); //refresh EEPROM id string to verify erase OK
        consolePrintSet(0,(start_line+1));
        consolePrintf("Copy Finished");
        return;
    }
    else
    {
        consolePrintSet(0,start_line);
        consolePrintf("Copy Failed!\n"
                      "eeprom size invalid!");
         return;
    }
    return;
}

void formatEEP(void)
{
    int test = areYouSure();
    if(test==1)
    {
        consolePrintSet(0,start_line);
        consolePrintf("Formatting EEPROM");

        int type = cardmeGetType();    //-1    0    1    2      3      get type and size of eeprom
        int length = cardmeSize(type); //0, 8192, 512, 8192, 256*1024

        char clear_CH[length]; //create array to copy to eeprom
        memset(clear_CH, 0xFF, sizeof(clear_CH));//clear the blank save buffer for 64k/4k EEPROM erase

        if((type == 1) || (type == 2))
        {
            cardmeWriteEeprom(0, (uint8 *)clear_CH, length, type); //copy FF buffer to EEPROM
            ref_eeprID(); //refresh EEPROM id string to verify erase OK
            consolePrintSet(0,(start_line+1));
            consolePrintf("Format Finished");
        }
        else if (type==3)  // erases type 3 (2mbit) eeproms
        {
            cardmeChipErase();
            consolePrintSet(0,(start_line+1));
            consolePrintf("Format Finished");
        }
        else
        {
            consolePrintSet(0,start_line);
            consolePrintf("Format Failed!\n"
                          "eeprom size invalid!");
        }
    }
    return;
}

static void mymemcpy(char* dest, char const* src, int size) 
{
  while(size--) 
    *dest++ = *src++;
}

void ref_eeprID(void)  //refreshes the EEPROM "ID string"
{
    char eepr_ID[12] = { 0,0,0,0,0,0,0,0,0,0,0,0 };
                                    //    -1         0            1           2           3
    char eepr_type[5][10] = { "No Card  ","Unknown  "," 4k eep  ","64k eep  ","2mbit eep"};

    int type = cardmeGetType();    //-1    0    1    2      3      get type and size of eeprom
    if (type >= 1)
    {
        cardmeReadEeprom(0, (u8 *)eepr_ID, (sizeof(eepr_ID) -1), type); // get the eeprom ID string

        consolePrintSet(0,2);
        consolePrintf("EEprom Type:                       ");
        consolePrintSet(0,2);
        consolePrintf("EEprom Type: %s", eepr_type[(type+1)]);
        consolePrintSet(0,3);
        consolePrintf("EEPROM \"ID\":                     ");
        consolePrintSet(0,3);
        consolePrintf("EEPROM \"ID\":%s", eepr_ID);
        consolePrintSet(0,4);
        consolePrintf("                                   ");

    }
    else
    {
        consolePrintSet(0,3);
        consolePrintf("EEPROM \"ID\":                     ");
        consolePrintSet(0,3);
        consolePrintf("EEPROM \"ID\": invalid");
    }
    return;
}

void ref_sramID(void)  //refreshes the SRAM "ID string"
{
    WAIT_CR &= ~0x0880;
    char sram_ID[12] = { 0,0,0,0,0,0,0,0,0,0,0,0 };

    mymemcpy(sram_ID, (char*)SRAM, sizeof(sram_ID) - 1);
    consolePrintSet(0,0);
    consolePrintf("                                ");
    consolePrintSet(0,0);
    consolePrintf("  SRAM Version ID: %sx   ", sram_ID);
    consolePrintf("--------------------------------");

    return;
}

void writeSram(int offset, char const* src, int size)
{
    WAIT_CR &= ~0x0880;
    char* dest = (char*)SRAM;
    if(offset>0){while(offset--){*dest++;}}
    int x = 0;
    while(size--)
    {*dest++ = (uint8)(src[x]); x++;}
}

void readSram(int offset, char* dest, int size)
{
    WAIT_CR &= ~0x0880;
    char* src = (char*)SRAM;
    if(offset>0){while(offset--){*src++;}}
    int x = 0;
    while(size--)
    {*src++ = *dest++; x++;}
}

void formatSram(void)
{
    //mymemcpy(smap, (char*)SRAM, size_smap);
    writeSram(0, smap, size_smap);
    return;
}

char app_ID[10]={'\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'};
// Return true if the card id is for etool
static int is_this_app() 
{
  return (
    app_ID[0] == 'E' &&
    app_ID[1] == 'T' &&
    app_ID[2] == 'o' &&
    app_ID[3] == 'o' &&
    app_ID[4] == 'l' &&
    app_ID[5] == '_' &&
    app_ID[6] == '1' &&
    app_ID[7] == '.' &&
    app_ID[8] == '0');
}

void checkFormat(void)
{
    WAIT_CR &= ~0x0880;
    mymemcpy(app_ID, (char*)SRAM, 9);

    consoleClear();
    consolePrintSet(0,0);
    consolePrintf("         EToolSMS 1.01          ");
    consolePrintf("--------------------------------");
    consolePrintf(" SRAM is not ready\n");
    consolePrintf(" do you wish to format it now?\n\n");
    consolePrintf(" press START+SELECT to format\n");
    consolePrintf(" or shutdown your DS.");
    consolePrintSet(0,20);
    consolePrintf("WARNING: SELECTING YES WILL     ");
    consolePrintf("ERASE YOUR GBA CART SRAM!\n");

    while (!is_this_app())
    {
        key_poll();
        if(((key_hit(KEY_START))||key_held(KEY_START))&&(key_hit(KEY_SELECT)||key_held(KEY_SELECT)))
        {
            //char ID[]="ETool_1.0\0";
            consolePrintSet(0,15);
            consolePrintf("FORMATTING PLEASE WAIT\n");
            consolePrintf("AppID: %s", app_ID);
            //consolePrintf("ID   : %s", ID);
            formatSram();
            //writeSram(0, ID , 9);
            mymemcpy(app_ID, (char*)SRAM, 9);
        }
    }
    consoleClear();
    return;
}
