#include <stdio.h>
#include <string.h>
#include <nds/memory.h>

#include <nds/arm9/console.h>
#include "menu.h"
#include "inout.h"
#include "bank_sel.h"
#include "Gdefines.h"
#include "cardme.h"
#include "banks.h"

#define main_delay 5000

void mainMenu(void)
{
    checkFormat();
    int result = 0;
    int choice = 1;
    drawMain(choice);

    while (1)
    {
        result=doMenu(5,choice);
        if (result == 1)
        {
            switch(choice)
            {
                case 1:   //backup
                    theWait();
                    consoleClear();
                    backupMenu();
                    drawMain(choice);
                    break;
                case 2:   //restore
                    theWait();
                    consoleClear();
                    restoreMenu();
                    drawMain(choice);
                    break;
                case 3:   //rename
                    theWait();
                    consoleClear();
                    renameMenu();
                    drawMain(choice);
                    break;
                case 4:   //format
                    theWait();
                    consoleClear();
                    formatMenu();
                    drawMain(choice);
                    break;
                case 5:   //info
                    infoMenu();
                    drawMain(choice);
                    break;
            }
        }
        else
        {
            consoleClear();
            drawMain(choice);
        }
    }
    return;
}

int areYouSure(void)
{
    clearIt();
    consolePrintSet(0,start_line+3);
    consolePrintf("         Are you sure?         \n");
    consolePrintf("       A - yes    B - no       ");
    while(1)
    {
        key_poll();
        // A Pressed - Yes
        if(key_hit(KEY_A))
        {
            clearIt();
            return 1;
        }
        // B Pressed - No
        if(key_hit(KEY_B)) 
        {
            clearIt();
            return 2;
        }
    }
}


void clearIt(void)
{
    consolePrintSet(0,start_line);
    consolePrintf("                                ");
    consolePrintf("                                ");
    consolePrintf("                                ");
    consolePrintf("                                ");
    consolePrintf("                                ");
    consolePrintf("                                ");
    consolePrintf("                                ");
    consolePrintf("                                ");
    return;
}

void infoMenu(void)
{
    consoleClear();
    consolePrintSet(0,4);
    consolePrintf("       EToolSMS 1.01\n");
    consolePrintf("         by cory1492\n\n");
    consolePrintf("     http://nds.cmamod.com\n\n");
    consolePrintf("     Thanks SATA!");
    consolePrintSet(0,21);
    consolePrintf("      Press A to continue");


    while(1)
    {
        key_poll();
        // A Pressed - select option
        if(key_hit(KEY_A))
        {
            consoleClear();
            return;
        }
    }
    return;
}

void drawMain(int loc)
{
    ref_sramID();
    ref_eeprID();

    consolePrintSet(0,5);
    consolePrintf("    Backup\n"
                  "    Restore\n"
                  "    Rename\n"
                  "    Format\n"
                  "    Info");
    consolePrintSet(0,22);
    consolePrintf("Use at your own risk.");
    consolePrintSet(0,(loc+4));
    consolePrintf("-->");

    return;
}

void backupMenu(void)
{
    int choice=0;
    int type = cardmeGetType();
    if (type == 1) //goto 4k select menu
    {
        choice = oneSel();
        if (choice == 0){consoleClear();return;}
        backupEEP(loc4[choice-1]);
//        chooseNewName(offset4[choice-1]);
        consoleClear();
        return;
    }

    else if (type == 2) //goto 64k select menu
    {
        choice = twoSel();
        if (choice == 0){consoleClear();return;}
        backupEEP(loc64[choice-1]);
//        chooseNewName(offset64[choice-1]);
        consoleClear();
        return;
    }
    return;
}

void restoreMenu(void)
{
    int choice = 0;
    int type = cardmeGetType();
    if (type == 1) //goto 4k select menu
    {
        choice = oneSel();
        if (choice == 0){consoleClear();return;}
        restoreEEP(loc4[choice-1]);
        consoleClear();
        return;
    }

    else if (type == 2) //goto 64k select menu
    {
        choice = twoSel();
        if (choice == 0){consoleClear();return;}
        restoreEEP(loc64[choice-1]);
        consoleClear();
        return;
    }
    return;
}

int doMenu(int menuLn, int& ch)
{
    int delay = main_delay;
    int delay2 = 0;
    consolePrintSet(0,(ch+4));
    consolePrintf("-->");
    while (1)
    {
        key_poll();

        // A Pressed - select option
        if(key_hit(KEY_A))
        {
            return 1;
        }

        // B pressed - go back
        if(key_hit(KEY_B))
        {
            return 0;
        }

        // Up Pressed
        if(key_hit(KEY_UP) && (delay == 0)) 
        {
            consolePrintSet(0,(ch+4));
            consolePrintf("   ");
            if (ch==1){ ch = menuLn; }
            else { ch = ch - 1;}
            consolePrintSet(0,(ch+4));
            consolePrintf("-->");
            delay = main_delay;
        }

        // Down Pressed 
        if(key_hit(KEY_DOWN) && (delay == 0)) 
        {
            consolePrintSet(0,(ch+4));
            consolePrintf("   ");
            if (ch == menuLn){ ch=1; }
            else { ch = ch + 1; }
            consolePrintSet(0,(ch+4));
            consolePrintf("-->");
            delay = main_delay;
        }
        consolePrintSet(0,20);
        consolePrintf("     ");
        if (delay>0){delay = delay - 1; delay2 = delay;}
    }
}

void waitIt(void)
{
    while(1)
    {
        key_poll();
        // A Pressed
        if(key_hit(KEY_A))
        {
            return;
        }
    }
    return;
}

void renameMenu(void)
{
    int choice = 0;
    int type = pickSize();
    if (type == 1) //goto 4k select menu
    {
        choice = oneSel();
        if (choice == 0){consoleClear();return;}
        renameIt(offset4[choice-1]);
        consoleClear();
        return;
    }
    else if (type == 2) //goto 64k select menu
    {
        choice = twoSel();
        if (choice == 0){consoleClear();return;}
        renameIt(offset64[choice-1]);
        consoleClear();
        return;
    }
    consoleClear();
    return;
}

int pickSize(void)
{
    consoleClear();
    consolePrintSet(0,1);
    consolePrintf("   Pick which type to rename    ");
    consolePrintf("--------------------------------");

    consolePrintSet(0,5);
    consolePrintf("     4k names\n"
                  "    64k names\n"
                  "    quit");
    theWait();
    int ret = 1;
    int choice = doMenu(3, ret);
    if (choice == 0){consoleClear();return 0;}
    else{consoleClear();return ret;}
}

#define SPACER 8
void renameIt(int offset)
{
    char name[]="                \0";
    int namepos=0; //which letter is being changed in array x-late to pos (9,5) for 0
    int locx = 0;//which array - caps/sm/special
    int locy = 0;//which char of array
    int tim=0;
    namingMenu();
    showtheKeys((namepos+SPACER), locx, locy);

    while (1)
    {
        key_poll();
        consolePrintSet(0,8);
        consolePrintf("New Name: %s", name);
        // A Pressed - put letter
        if(key_hit(KEY_A))
        {
            name[namepos]=letters[locx][locy];
            if (namepos < 15){hidetheKeys(namepos + SPACER, 0); namepos = namepos + 1;}
            showtheKeys((namepos+SPACER), locx, locy);
        }

        // B  or left pressed - backspace
        if(key_hit(KEY_B)||key_hit(KEY_LEFT))
        {
            if(namepos > 0)
            {
                hidetheKeys(namepos+SPACER, 1);
                
                namepos = namepos - 1;
                name[namepos] = ' ';
                showtheKeys((namepos+SPACER), locx, locy);
            }
        }

        // right pressed - space
        if(key_hit(KEY_RIGHT))
        {
            name[namepos]=' ';
            hidetheKeys(namepos + SPACER, 0); 
            consolePutChar((namepos+SPACER), 5, ' ');
            if (namepos < 15)
            {
                namepos = namepos + 1;
                showtheKeys((namepos+SPACER), locx, locy);
            }
            else if (namepos == 15)
            {
               showtheKeys((namepos+SPACER), locx, locy);
            }
        }

        // UP Pressed  || key_held(KEY_UP)
        if(key_hit(KEY_UP) || (key_held(KEY_UP) && tim==0)) 
        {
            if(locy == 0){locy = 25;}
            else{locy = locy - 1;}
            showtheKeys((namepos+SPACER), locx, locy);
            tim=3500;
        }

        // Down Pressed || key_held(KEY_DOWN)
        if(key_hit(KEY_DOWN) || (key_held(KEY_DOWN) && tim==0) ) 
        {
            if(locy == 25){locy = 0;}
            else{locy = locy + 1;}
            showtheKeys((namepos+SPACER), locx, locy);
            tim=3500;
        }

        // R Pressed
        if(key_hit(KEY_R)) 
        {
           if(locx == 2){locx = 0;}
           else{locx = locx + 1;}
           showtheKeys((namepos+SPACER), locx, locy);
        }

        // Start Pressed
        if(key_hit(KEY_START)) 
        {
            WAIT_CR &= ~0x0880;
            writeSram(offset, name, 16);
            consoleClear();
            return;
        }

        // Select Pressed
        if(key_hit(KEY_SELECT)) 
        {
            consoleClear();
            return;
        }
        if (tim>0){tim--;}
    }
    return;
}

void namingMenu(void)
{
    consoleClear();
    consolePrintSet(0,1);
    consolePrintf("         Enter the name         ");
    consolePrintf("--------------------------------");
    consolePrintSet(0,5);
    consolePrintf("        ----------------        "); //16 characters for name

    consolePrintSet(0,17);
    consolePrintf("     A - select letter\n");
    consolePrintf("     B - backspace\n");
    consolePrintf("R SHLD - switch  CAP/small/#\n");
    consolePrintf(" up/dn - change character\n");
    consolePrintf(" Start - save\nSelect - quit");
    return;
}

void showtheKeys(int pos, int x, int y)
{
    char c,d,e;

    if (y == 0){c = letters[x][25];} //above
    else{c = letters[x][(y-1)];}

    d = letters[x][y]; //at position

    if (y == 25){e = letters[x][0];} // below
    else{e = letters[x][(y+1)];}

    consolePutChar(pos, 4, c); //above
    consolePutChar(pos, 5, d); //at
    consolePutChar(pos, 6, e); //below
    return;
}

void hidetheKeys(int pos, bool middle)
{
    consolePutChar((int)pos, 4, ' ');
    if (middle){consolePutChar(pos, 5, '-');}
    consolePutChar(pos, 6, ' ');
    return;
}

/* not working right - erases what it reads?
void chooseNewName(int offset)
{
    int ret = 1;
    int choice = 0;

    consoleClear();
    consolePrintSet(0,0);
    consolePrintf("Backup done! Rename savename?\n");
    consolePrintf("--------------------------------");

    consolePrintSet(0,5);
    consolePrintf("    yes\n"
                  "    no\n");
    theWait();
    choice = doMenu(2, ret);
    if (ret == 2 || choice == 0) {return;}

    if (ret == 1 && choice == 1)
    {
        renameIt(offset);
        return;
    }
    else return;
}
*/

void formatMenu (void)
{
    ref_sramID();
    ref_eeprID();
    consoleClear();
    consolePrintSet(0,1);
    consolePrintf("  Which will you erase/format?  ");
    consolePrintf("--------------------------------");
    consolePrintSet(0,5);
    consolePrintf("    EEPROM\n"
                  "    SRAM\n"
                  "    back");
    int ret = 1;
    int choice = doMenu(3, ret);
    if (ret == 1 && choice == 1) {formatEEP(); consoleClear(); return;}
    else if (ret == 2 && choice == 1)
    {
        ret = areYouSure();
        if (ret==1) {formatSram();}
        consoleClear();
        return;
    }
    return;
}

void theWait(void)
{
    for (int i=0; i < 7000; i++)
    {   int x = 50+50; x=x+10; }
    return;
}
